#pragma once

#include <Object.hh>
#include <TaskInfo.pb.h>
#include <json/json.h>
namespace Salsa {
///
/// \class Job
///
/// \brief Job class
/// \author Matej Fedor <matej.fedor.mf@gmail.com>
/// \author Martin Vala <mvala@cern.ch>
/// \author Branislav Beke <bbeke@badhaven.dev>
///

class Job : public Object {
public:
    /// Queue types
    enum QueueType { pending, assigned, running, done, failed, all }; // TODO maybe go explicit?
    Job(std::string uuid = "", std::string type = "NONE");
    virtual ~Job();

    void print() const;
    void json(Json::Value & json);

    /// TODO Get next available task?job?
    TaskInfo * nextJob(); // I assume task

    // TODO Annotate task ops
    void tasks(std::vector<TaskInfo *> & v, QueueType type, bool clear = true);
    bool addTask(uint32_t id, TaskInfo * pJob, QueueType type);
    bool moveTask(uint32_t id, QueueType from, QueueType to);
    bool moveTask(uint32_t id, TaskInfo * pJI, QueueType from, QueueType to);
    bool removeTask(uint32_t id, QueueType from);

    // TODO size ops
    size_t size(QueueType t = all) const;
    size_t sizeNotFinished() const;

    // TODO no idea...
    void        consumer(std::string uuid);
    std::string consumer() const;

    // TODO no idea #2...
    void        feeder(std::string uuid);
    std::string feeder() const;

    // TODO task statuses
    bool haveMoreTasks() const;
    void haveMoreTasks(bool hasMoreTasks);

    /// Check task presence in certain queue
    bool isTaskInQueue(uint32_t id, QueueType type) const;

protected:
    std::map<uint32_t, TaskInfo *> mTasks[all]    = {};     ///< Lists of jobs
    std::string                    mUUID          = "";     ///< Job UUID
    std::string                    mConsumerUUID  = "";     ///< Source (consumer) UUID
    std::string                    mFeederUUID    = "";     ///< Feeder UUID
    std::string                    mType          = "NONE"; ///< Job type
    bool                           mHaveMoreTasks = true;   ///< Flag if we have more tasks
};
} // namespace Salsa
