#pragma once
#include <Consumer.hh>
#include <Feeder.hh>
#include <Job.hh>
#include <Object.hh>
#include <Publisher.hh>
#include <Socket.hh>
#include <TaskPool.hh>
#include <Worker.hh>
namespace Salsa {
///
/// \class NodeManager
///
/// \brief NodeManager class
/// \author Matej Fedor <matej.fedor.mf@gmail.com>
/// \author Martin Vala <mvala@cern.ch>
/// \author Branislav Beke <bbeke@badhaven.dev>
///

class NodeManager : public Object {
public:
    NodeManager();
    virtual ~NodeManager();

    void print(std::string opt = "") const;

    // TODO Annotate add ops
    void addConsumer(std::string uuid, std::shared_ptr<Socket> s);
    void addFeeder(std::string uuid, std::shared_ptr<Socket> s);
    void addWorker(std::string uuid, std::shared_ptr<Socket> s);
    void addTask(TaskInfo * taskInfo, std::string cuuid, std::string fuuid,
                 Salsa::Job::QueueType t = Salsa::Job::pending);

    // TODO Yet again...
    virtual Socket * onEnter(std::string self, std::string fromType, Message * msg, std::vector<std::string> & out);
    virtual Socket * onExit(std::string self, Message * msg, std::vector<std::string> & out);
    virtual Socket * onWhisper(std::string self, Message * msg, std::vector<std::string> & out);

    // TODO Return ops
    std::shared_ptr<Feeder>   feeder(std::string uuid) const;
    std::shared_ptr<Consumer> consumer(std::string uuid) const;
    std::shared_ptr<Worker>   worker(std::string uuid) const;
    Job *                     job(std::string uuid);

    /// Get NM's task pool
    TaskPool * taskPool();

    // TODO Annotate
    virtual void addTaskSlot();
    bool         hasJobs() const;

    int32_t nSlots(double mult = 1.0) const;
    void    jobs(std::string clientUUID, std::vector<std::string> & jobs) const; // TODO ehm, what?

    // TODO Task ops
    TaskInfo * getNextTask();
    void       resultTask(TaskInfo * task);
    void       noMoreTasks(std::string jobUUID);
    bool       haveMoreTasks();
    bool       haveMoreTasks(std::string jobUUID);

    /// Run task interface
    virtual void runTask(TaskState * ts, std::string wk, std::string upstream) = 0;

    // TODO Terminate ops
    virtual void terminateJob(std::string uuid);
    virtual void terminateJobAll(); // TODO Why not terminateAllJobs() ? Seems more readable to me

    virtual bool handleTaskPool(void * p);
    virtual bool sendWhisper(Socket * s, std::string to, std::vector<std::string> & v);

    // TODO annotate ops
    virtual void        publisher(Publisher * p);
    virtual Publisher * publisher() const;
    virtual void        publish(std::string id) const; // TODO what does this do again? const and no return...

protected:
    std::map<std::string, Job *>                     mJobs{};               ///< List of jobs
    std::vector<std::string>                         mActiveJobs{};         ///< List of active jobs
    std::map<std::string, std::shared_ptr<Worker>>   mWorkers{};            ///< List of Workers
    std::map<std::string, std::shared_ptr<Consumer>> mConsumers{};          ///< List of Consumers
    std::map<std::string, std::shared_ptr<Feeder>>   mFeeders{};            ///< List of Feeders
    TaskPool *                                       mpTaskPool  = nullptr; ///< Task pool
    Publisher *                                      mpPublisher = nullptr; ///< Publisher
};
} // namespace Salsa
